# -*- coding: utf-8 -*-

"""Code to safely download storage bucket files."""

from attr import dataclass
from google.cloud.storage import Blob, Bucket, Client
from returns.functions import raise_exception
from returns.pipeline import pipeline
from returns.result import ResultE, safe
from typing_extensions import final


@final
@dataclass(frozen=True, slots=True)
class DownloadFile(object):
    """
    Downloades a google cloud storage bucket file.

    .. versionadded:: 0.1.0

    :param bucket_name: Name of the google cloud bucket
    :type bucket_name: str

    :param filename: The name of the file to download
    :type filename: str

    :param encoding: Optional, Encoding to use when decoding bytes, defaults
    to utf-8
    :type encoding: str

    :return: Success/Failure containers
    """

    _client = Client

    @pipeline(ResultE)
    def __call__(
        self,
        storage_bucket_name: str,
        filename: str,
    ) -> ResultE[bytes]:
        """Download storage bucket file."""
        client = self._initialize_client().unwrap()
        bucket = self._get_bucket(client, storage_bucket_name).unwrap()
        blob = self._get_blob(bucket, filename).unwrap()
        return self._get_bytes(blob)

    @safe
    def _initialize_client(self) -> object:
        return self._client()

    @safe
    def _get_bucket(self, client: Client, storage_bucket_name: str) -> object:
        return client.get_bucket(storage_bucket_name)

    @safe
    def _get_blob(self, bucket: Bucket, filename: str) -> object:
        return bucket.blob(filename)

    @safe
    def _get_bytes(self, blob: Blob) -> bytes:
        return blob.download_as_string()


def download_file(
    storage_bucket_name: str,
    filename: str,
) -> bytes:
    """Download file as per usual but raise exception on error."""
    return DownloadFile()(
        storage_bucket_name,
        filename,
    ).alt(
        raise_exception,
    ).unwrap()



# -*- coding: utf-8 -*-

"""Code to safely upload storage bucket files."""

from typing import Union

from attr import dataclass
from google.cloud.storage import Blob, Bucket, Client
from returns.functions import raise_exception
from returns.pipeline import pipeline
from returns.result import ResultE, safe
from typing_extensions import final


@final
@dataclass(frozen=True, slots=True)
class UploadFile(object):
    """
    Uploades contents in file_data to a google cloud storage bucket.

    .. versionadded:: 0.0.1

    :param file_data: contents to upload in bytes
    :type file_data: bytes

    :param bucket_name: Name of the google cloud bucket
    :type bucket_name: str

    :param filename: The name to give the uploaded file
    :type filename: str

    :param content_type: What type of file to create, defaults to text/plain
    :type content_type: str

    :return: Success/Failure containers
    """

    _client = Client

    @pipeline(ResultE)
    def __call__(  # noqa: WPS211
        self,
        file_data: bytes,
        storage_bucket_name: str,
        filename: str,
        content_type: str = 'application/octet-stream',
    ) -> ResultE[bool]:
        """Download storage bucket file."""
        client = self._initialize_client().unwrap()
        bucket = self._get_bucket(client, storage_bucket_name).unwrap()
        blob = self._get_blob(bucket, filename).unwrap()
        return self._upload_data(blob, file_data, content_type)

    @safe
    def _initialize_client(self) -> object:
        return self._client()

    @safe
    def _get_bucket(
        self, client: Client, storage_bucket_name: str,
    ) -> object:
        return client.get_bucket(storage_bucket_name)

    @safe
    def _get_blob(
        self, bucket: Bucket, filename: str,
    ) -> object:
        return bucket.blob(filename)

    @safe
    def _handle_file_data(
        self, file_data: Union[str, bytes], encoding: str,
    ) -> bytes:
        if isinstance(file_data, str):
            return file_data.encode(encoding)
        return file_data

    @safe
    def _upload_data(
        self, blob: Blob, file_content: bytes, content_type: str,
    ) -> bool:
        blob.upload_from_string(file_content, content_type)
        return True


def upload_file(
    file_data: bytes,
    storage_bucket_name: str,
    filename: str,
    content_type: str = 'application/octet-stream',
) -> bool:
    """Upload file as per usual but raise exception on error."""
    return UploadFile()(
        file_data,
        storage_bucket_name,
        filename,
        content_type,
    ).alt(
        raise_exception,
    ).unwrap()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['storage_bucket']

package_data = \
{'': ['*']}

install_requires = \
['attrs>=19.3.0,<20.0.0',
 'google-cloud-storage>=1.28.1,<2.0.0',
 'returns>=0.13.0,<0.14.0']

setup_kwargs = {
    'name': 'storage-bucket',
    'version': '0.1.1',
    'description': 'Easy to work with Google Cloud Platform Storage Bucket wrapper',
    'long_description': "# Google Cloud Platform Storage Bucket EaseyBreezey\n\nThis package just aims to make life a little bit easier for people who have to work with google cloud storage bucket.\n\n\n## Quickstart:\n\n1. get the package\n  * `pip install storage-bucket`\n2. Download your keyfile and save it as key.json and point to it with env var:\n  * `gcloud iam service-accounts keys create key.json --iam-account your_service_account@your_project.iam.gserviceaccount.com`\n  * `export GOOGLE_APPLICATION_CREDENTIALS='key.json'`\n3. Run some code:\n\n\n```python\nfrom storage_bucket.download_file import DownloadFile, download_file\n\n\ndef use_data_for_something(data):\n    print(data)\n\n# Returns Modal way\ndownloader = DownloadFile()\ndownloader(\n    'my_bucket',\n    'my_file.txt',\n).map(\n    use_data_for_something,  # send data to this function,\n).alt(\n    print,  # print error or send a mail or w/e\n)\n\n# Normal way, this might throw exception.\nmy_data = download_file(\n    'my_bucket',\n    'my_file.txt',\n)\nprint(my_data)\n```\n\n### The use of [Returns](https://github.com/dry-python/returns) library.\n  * Just lets us get rid of all exceptions.\n  * Lets us chain stuff so everything looks good.\n  * Lets you use `DownloadFile()(args...).bind(dostuff).alt(dostuffonfailure)`\n  * Don't like it? use the matching normal function provided for your convenience.\n\n\n",
    'author': 'Thomas Borgen',
    'author_email': 'thomas@borgenit.no',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-

"""Code to safely list storage bucket files."""

from typing import Optional, Set

from attr import dataclass
from google.cloud.storage import Blob, Client
from returns.functions import raise_exception
from returns.pipeline import pipeline
from returns.result import ResultE, safe
from typing_extensions import final


@final
@dataclass(frozen=True, slots=True)
class ListFiles(object):
    """List all blobs in given storage_bucket."""

    _client = Client

    @pipeline(ResultE)
    def __call__(
        self,
        storage_bucket_name: str,
        prefix: Optional[str] = None,
    ) -> ResultE[Set[Blob]]:
        """List the storage bucket files."""
        client = self._initialize_client().unwrap()  # type: ignore
        return self._list_blobs(
            client,
            storage_bucket_name,
            prefix,
        )

    @safe
    def _list_blobs(
        self,
        client: Client,
        storage_bucket_name: str,
        prefix: Optional[str] = None,
    ) -> Set[Blob]:
        return set(client.list_blobs(storage_bucket_name, prefix=prefix))

    @safe
    def _initialize_client(self) -> Client:
        return self._client()

    @safe
    def _get_bucket(
        self, client: Client, storage_bucket_name: str,
    ) -> object:
        return client.get_bucket(storage_bucket_name)


def list_files(
    storage_bucket_name: str,
    prefix: Optional[str] = None,
) -> Set[Blob]:
    """Download file as per usual but raise exception on error."""
    return ListFiles()(
        storage_bucket_name,
        prefix,
    ).alt(
        raise_exception,
    ).unwrap()
